@@ -1,7 +1,7 @@
 import os
+import re
 import datetime as dt
 import subprocess
-import unicodedata
 import struct
 import shutil
 import atexit
@@ -41,6 +41,9 @@ class Apps(object):
                 if pid_info != '"pid"=[ NULL ] \n':
                     yield int(pid_info.split('=')[1])
 
+    def keys(self):
+        return list(self._iter_excel_instances())
+
     def __iter__(self):
         for pid in self._iter_excel_instances():
             yield App(xl=pid)
@@ -48,9 +51,10 @@ class Apps(object):
     def __len__(self):
         return len(list(self._iter_excel_instances()))
 
-    def __getitem__(self, index):
-        pids = list(self._iter_excel_instances())
-        return App(xl=pids[index])
+    def __getitem__(self, pid):
+        if pid not in self.keys():
+            raise Exception('Could not find an Excel instance with this PID.')
+        return App(xl=pid)
 
 
 class App(object):
@@ -81,7 +85,11 @@ class App(object):
     @property
     def selection(self):
         sheet = self.books.active.sheets.active
-        return Range(sheet, self.xl.selection.get_address())
+        try:
+            # fails if e.g. chart is selected
+            return Range(sheet, self.xl.selection.get_address())
+        except CommandError:
+            return None
 
     def activate(self, steal_focus=False):
         asn = subprocess.check_output(['lsappinfo', 'visibleprocesslist', '-includehidden']).decode('utf-8')
@@ -408,6 +416,10 @@ class Sheet(object):
     def pictures(self):
         return Pictures(self)
 
+    @property
+    def used_range(self):
+        return Range(self, self.xl.used_range.get_address())
+
 
 class Range(object):
 
@@ -498,7 +510,8 @@ class Range(object):
     @property
     def formula_array(self):
         if self.xl is not None:
-            return self.xl.formula_array.get()
+            rv = self.xl.formula_array.get()
+            return None if rv == kw.missing_value else rv
 
     @formula_array.setter
     def formula_array(self, value):
@@ -508,7 +521,8 @@ class Range(object):
     @property
     def column_width(self):
         if self.xl is not None:
-            return self.xl.column_width.get()
+            rv = self.xl.column_width.get()
+            return None if rv == kw.missing_value else rv
         else:
             return 0
 
@@ -520,7 +534,8 @@ class Range(object):
     @property
     def row_height(self):
         if self.xl is not None:
-            return self.xl.row_height.get()
+            rv = self.xl.row_height.get()
+            return None if rv == kw.missing_value else rv
         else:
             return 0
 
@@ -554,7 +569,8 @@ class Range(object):
     @property
     def number_format(self):
         if self.xl is not None:
-            return self.xl.number_format.get()
+            rv = self.xl.number_format.get()
+            return None if rv == kw.missing_value else rv
 
     @number_format.setter
     def number_format(self, value):
@@ -1032,13 +1048,10 @@ class Name(object):
 
     @property
     def refers_to_range(self):
-        ref = self.refers_to[1:].split('!')
         book = self.parent if isinstance(self.parent, Book) else self.parent.book
-        # appscript has issues when there are blanks or ' in sheet names (e.g. "foo' bar")
-        sheetname = ref[0]
-        if sheetname.startswith("'") and sheetname.endswith("'"):
-            ref[0] = sheetname[1:-1].replace("''", "'")
-        return Range(Sheet(book, ref[0]), ref[1])
+        external_address = self.xl.reference_range.get_address(external=True)
+        match = re.search(r"\](.*)!(.*)", external_address)
+        return Range(Sheet(book, match.group(1)), match.group(2))
 
 
 class Shapes(Collection):
